/*============================================================================
  File:     TempdbSetup.sql

  Summary:  Stored proc to drive tempdb space usage

  Date:     March 2011

  SQL Server Versions:
		10.0.2531.00 (SS2008 SP1)
		9.00.4035.00 (SS2005 SP3)
------------------------------------------------------------------------------
  Written by Paul S. Randal, SQLskills.com

  For more scripts and sample code, check out 
    http://www.SQLskills.com

  You may alter this code for your own *non-commercial* purposes. You may
  republish altered code as long as you give due credit.
  
  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF 
  ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED 
  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
  PARTICULAR PURPOSE.
============================================================================*/

USE msdb
GO

IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[MyStoredProc]'))
DROP PROCEDURE [dbo].[MyStoredProc]
GO

CREATE PROCEDURE MyStoredProc
AS
BEGIN

-- Pre-aggregate data into tempdb
SELECT * INTO #TempCustomers
FROM SalesDB..Customers;

CREATE NONCLUSTERED INDEX #TC_CustomerID
ON #TempCustomers (CustomerID);

SELECT * INTO #TempProducts
FROM SalesDB..Products;

CREATE NONCLUSTERED INDEX #TP_ProductID
ON #TempProducts (ProductID);
CREATE NONCLUSTERED INDEX #TP_Price
ON #TempProducts (Price);


SELECT * INTO #TempSales
FROM SalesDB..Sales;

CREATE NONCLUSTERED INDEX #TS_SalesID
ON #TempSales (SalesID);
CREATE NONCLUSTERED INDEX #TS_CustomerID
ON #TempSales (CustomerID);
CREATE NONCLUSTERED INDEX #TS_ProductID
ON #TempSales (ProductID);


SELECT
	tp.Name AS [Product],
	SUM (ts.Quantity) AS [Quantity],
	tp.Price AS [Amount]
FROM #TempProducts AS tp
JOIN #TempSales AS ts
	ON ts.ProductID = tp.ProductID
GROUP BY tp.Name, tp.Price
ORDER BY tp.Name;
END
GO


EXEC msdb..MyStoredProc;
GO
